/**
 * For test purposes, this class extracts as much information as
 * possible from a ContentHandler. It's useful to compare what
 * a parser generates against what a class of mine does.
 *
 * @author <a href="http://www.marchal.com">Beno&#238;t Marchal</a>
 */

// Copyright 2001, Benot Marchal.

package org.ananas.util;

import java.io.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

public class ContentHandlerExtractor
   implements ContentHandler
{
   /**
    * writes the result here
    */
   protected PrintWriter writer;

   /**
    * creates a ContentHandlerExtractor that writes to System.out
    */
   public ContentHandlerExtractor()
   {
      writer = new PrintWriter(new OutputStreamWriter(System.out));
   }

   /**
    * creates a ContentHandlerExtractor that writes to the given PrintWriter
    * @param writer PrintWriter to write to
    */
   public ContentHandlerExtractor(PrintWriter writer)
   {
      this.writer = writer;
   }

   /**
    * convenience: creates a ContentHandlerExtractor that writes to a Writer
    * @param writer Writer to write to
    */
   public ContentHandlerExtractor(Writer writer)
   {
      this(new PrintWriter(writer));
   }

   /**
    * writes as much data as possible
    * @throw SAXException IO error
    */
   public void characters(char[] ch,int start,int length)
      throws SAXException
   {
      writer.print("* characters:            ch='");
      writer.write(ch,start,length);
      writer.print("' start=");
      writer.print(start);
      writer.print(" length=");
      writer.println(length);
   }

   /**
    * writes as much data as possible
    */
   public void endDocument()
   {
      writer.println("* endDocument");
      writer.flush();
   }

   /**
    * writes as much data as possible
    */
   public void endElement(String namespaceURI,String lName,String qName)
   {
      writer.print("* endElement:            namespaceURI='");
      writer.print(namespaceURI);
      writer.print("' localName='");
      writer.print(lName);
      writer.print("' qualifiedName='");
      writer.print(qName);
      writer.println('\'');
   }

   /**
    * writes as much data as possible
    */
   public void endPrefixMapping(String prefix)
   {
      writer.print("* endPrefixMapping:      prefix='");
      writer.print(prefix);
      writer.println('\'');
   }

   /**
    * writes as much data as possible
    * @throw SAXException IO error
    */
   public void ignorableWhitespace(char[] ch,int start,int length)
      throws SAXException
   {
      writer.print("* ignorableWhitespace:   ch='");
      writer.write(ch,start,length);
      writer.print("' start=");
      writer.print(start);
      writer.print(" length=");
      writer.println(length);
   }

   /**
    * writes as much data as possible
    */
   public void processingInstruction(String target,String data)
   {
      writer.print("* processingInstruction: target='");
      writer.print(target);
      writer.print("' data='");
      writer.print(data);
      writer.println('\'');
   }

   /**
    * writes as much data as possible
    */
   public void setDocumentLocator(Locator locator)
   {
      writer.print("* setDocumentLocator:    publicId='");
      writer.print(locator.getPublicId());
      writer.print("' systemId='");
      writer.print(locator.getSystemId());
      writer.println('\'');
   }

   /**
    * writes as much data as possible
    */
   public void skippedEntity(String name)
   {
      writer.print("* skippedEntity:         name='");
      writer.print(name);
      writer.println('\'');
   }

   /**
    * writes as much data as possible
    */
   public void startDocument()
   {
      writer.println("* startDocument");
   }

   /**
    * writes as much data as possible
    */
   public void startElement(String namespaceURI,String lName,String qName,Attributes attributes)
   {
      writer.print("* startElement:          namespaceURI='");
      writer.print(namespaceURI);
      writer.print("' localName='");
      writer.print(lName);
      writer.print("' qualifiedName='");
      writer.print(qName);
      writer.print("' attributesLength='");
      writer.print(attributes.getLength());
      writer.println('\'');
      for(int i = 0;i < attributes.getLength();i++)
      {
         writer.print("  attribute:             namespaceURI='");
         writer.print(attributes.getURI(i));
         writer.print("' localName='");
         writer.print(attributes.getLocalName(i));
         writer.print("' qualifiedName='");
         writer.print(attributes.getQName(i));
         writer.print("' index='");
         writer.print(i);
         writer.print("' type='");
         writer.print(attributes.getType(i));
         writer.print("' value='");
         writer.print(attributes.getValue(i));
         writer.println('\'');
      }
   }

   /**
    * writes as much data as possible
    */
   public void startPrefixMapping(String prefix,String uri)
   {
      writer.print("* startPrefixMapping:    prefix='");
      writer.print(prefix);
      writer.print("' uri='");
      writer.print(uri);
      writer.println('\'');
   }

   /**
    * entry-point for test purposes only, it parses a document
    * after registering a ContentHandlerExtractor with the parser
    * @param args command-line arguments
    */
   public static void main(String[] args)
      throws SAXException, IOException
   {
      XMLReader reader = null;
      try
      {
         reader = XMLReaderFactory.createXMLReader();
      }
      catch(SAXException e)
      {
         reader = XMLReaderFactory.createXMLReader("org.apache.xerces.parsers.SAXParser");
      }
      ContentHandler contentHandler = null;
      if(args.length == 1)
         contentHandler = new ContentHandlerExtractor();
      else
         contentHandler = new ContentHandlerExtractor(new FileWriter(args[1]));
      reader.setFeature("http://xml.org/sax/features/namespace-prefixes",true);
      reader.setContentHandler(contentHandler);
      reader.parse(args[0]);
   }
}
