/**
 * Offers handy services to the JAXP API. This class implements
 * the "missing" methods in JAXP, i.e. those combination I use
 * often and I' would have liked to see in JAXP.
 *
 * @author <a href="http://www.marchal.com">Beno&#238;t Marchal</a>
 */

// Copyright 2001, Benot Marchal.

package org.ananas.util;

import java.io.*;
import org.xml.sax.*;
import javax.xml.parsers.*;
import org.xml.sax.helpers.XMLReaderFactory;

public class JAXPHelper
{
   /**
    * Constant for the SAX lexical handler property.
    */
   public static final String LEXICAL_HANDLER =
      "http://xml.org/sax/properties/lexical-handler";

   /**
    * Constants to disable loading the external DTD...
    * Hopefully one of them will work (they vary depending on the
    * parser so I need to try them all).
    */
   private static final String[] DISABLE_DTD =
   {
      "http://xml.org/sax/features/external-parameter-entities",
      "http://apache.org/xml/features/nonvalidating/load-external-dtd",
      "http://apache.org/xml/features/nonvalidating/load-dtd",
   };

   /**
    * Creates an XMLReader through JAXP (i.e. it knowns the
    * appropriate class for the installed library).
    * @param namespaceAware makes the reader namespace aware
    * @param loadExternalDTD tells the reader to ignore the external
    *    DTD completely (the reader is always non-validating)
    *    i.e. non-validating and, as far the parser permits,
    *         does not read external the DTD at all, even if it declares
    *         entities and other useful stuffs
    * @return initialized XMLReader
    * @throw SAXException if it fails to create the reader
    */
   public static XMLReader createXMLReader(boolean namespaceAware,boolean loadExternalDTD)
      throws SAXException
   {
      XMLReader reader = null;
      try
      {
         SAXParserFactory parserFactory = SAXParserFactory.newInstance();
         parserFactory.setNamespaceAware(namespaceAware);
         parserFactory.setValidating(false);
         SAXParser parser = parserFactory.newSAXParser();
         reader = parser.getXMLReader();
         if(!loadExternalDTD)
         {
            SAXException x;
            int i = 0;
            do
            {
               try
               {
                  reader.setFeature(DISABLE_DTD[i],false);
                  // if there's an exception, x is not set to null
                  x = null;
               }
               catch(SAXNotRecognizedException e)
               {
                  x = e;
               }
               catch(SAXNotSupportedException e)
               {
                  x = e;
               }
            }
            while(x != null && ++i < DISABLE_DTD.length);
/*            if(x != null)
               throw x;*/
         }
      }
      catch(FactoryConfigurationError e)
      {
         throw new SAXException("FactoryConfigurationError: " + e.getMessage());
      }
      catch(ParserConfigurationException e)
      {
         throw new SAXException(e);
      }
      if(null == reader)
      {
         reader = XMLReaderFactory.createXMLReader();
         reader.setFeature("http://xml.org/sax/features/namespaces",namespaceAware);
      }
      return reader;
   }

   /**
    * Creates an XMLReader through JAXP (i.e. it knowns the
    * appropriate class for the installed library).
    * Equivallent to calling createXMLReader(namespaceAware,true)
    * @param namespaceAware makes the reader namespace aware
    * @return initialized XMLReader
    * @throw SAXException if it fails to create the reader
    */
   public static XMLReader createXMLReader(boolean namespaceAware)
      throws SAXException
   {
      return createXMLReader(namespaceAware,true);
   }

   /**
    * Creates a namespace-aware XMLReader through JAXP. Equivallent
    * to calling createXMLReader(true).
    * @return initialized XMLReader
    * @throw SAXException if it fails to create the reader
    */
   public static XMLReader createXMLReader()
      throws SAXException
   {
      return createXMLReader(true);
   }

   /**
    * One of the missing constructor for InputSource, turns a
    * File object into an InputSource object.
    * @param file to create the InputSource with
    * @return InputSource that accesses the file
    * @throw IOException in case of error
    */
   public static InputSource toInputSource(File file)
      throws IOException
   {
      InputSource source = new InputSource(new FileInputStream(file));
      source.setSystemId(file.getPath());
      return source;
   }
}
