/**
 * Default implementation for the messager. It prints most
 * error messages on a PrintWriter. See each method description
 * for more details. It flushes the writer after each writing operation.
 *
 * @author <a href="http://www.marchal.com">Beno&#238;t Marchal</a>
 */

// Copyright 2001, Benot Marchal.

package org.ananas.xm;

import java.io.*;
import java.text.MessageFormat;

public class DefaultMessenger
   implements Messenger
{
   /**
    * PrintWriter this messager outputs to
    */
   protected PrintWriter writer;

   /**
    * Creates a DefaultMessenger that prints on the console.
    */
   public DefaultMessenger()
   {
      writer = new PrintWriter(new OutputStreamWriter(System.out));
   }

   /**
    * Creates a DefaultMessenger that prints on the given writer.
    * @param writer writer to use for output
    */
   public DefaultMessenger(PrintWriter writer)
   {
      this.writer = writer;
   }

   /**
    * Writes the exception's message and throws the exception.
    * @param e exception describing the error
    */
   public void error(XMException e)
      throws XMException
   {
      throw e;
   }

   /**
    * Writes the exception's message and throws the exception.
    * @param e exception describing the error
    */
   public void fatal(XMException e)
      throws XMException
   {
      throw e;
   }

   /**
    * Writes the warning message on the writer. Does not throw the exception.
    * @param e exception describing the error
    */
   public void warning(XMException e)
      throws XMException
   {
      writer.println(e.getLocalizedMessage());
      writer.flush();
   }

   /**
    * Progress indicator: a file has been processed. If no result
    * file was created, resultFile is null.
    * @param sourceFile the file just processed
    * @param resultFile the file just created or null if no result
    *   (e.g. the result file already exists)
    * @return always true, the default implementation does not
    *    give the user a "Cancel" option
    */
   public boolean progress(File sourceFile,File resultFile)
   {
      if(resultFile != null)
         info(Resources.getString("progressdone"),
              new Object[] {sourceFile.getPath(), resultFile.getPath()});
      else
         info(Resources.getString("progressignored"),
              new Object[] {sourceFile.getPath()});
      return true;
   }

   /**
    * Writes a progress message on the writer.
    * @param file file that was last ignored
    */
   public void ignored(File file)
   {
      info(Resources.getString("ignored"),
           new Object[] {file.getAbsolutePath()});
   }

   /**
    * Writes the message on the writer.
    * @param msg exception describing the error
    */
   public void info(String msg)
   {
      writer.println(msg);
      writer.flush();
   }

   /**
    * Writes the message on the writer.
    * @param pattern message pattern
    * @param arguments message arguments
    */
   public void info(String pattern,Object[] arguments)
   {
      writer.println(MessageFormat.format(pattern,arguments));
      writer.flush();
   }
}
