/**
 * This filter fixes hyperlinks in the output document. So far,
 * it only recognizes HTML and XHTML.
 *
 * @author <a href="http://www.marchal.com">Beno&#238;t Marchal</a>
 */

// Copyright 2001, Benot Marchal.

package org.ananas.xm;

import java.io.*;
import java.net.*;
import java.util.*;
import org.xml.sax.*;
import org.ananas.util.*;
import org.xml.sax.helpers.*;
import org.xml.sax.ext.LexicalHandler;

public class LinkFilter
   extends XMLFilterImpl
   implements LexicalHandler
{
   /**
    * reports errors, warnings, etc. on this messenger
    */
   protected Messenger messenger;

   /**
    * selects the right mover for the job
    */
   protected MoversSupervisor supervisor;

   /**
    * the prefix that will lead to the root
    */
   protected String rootPrefix = null;

   /**
    * the current directory
    */
   protected File directory;

   /**
    * the lexical handler, if present
    */
   protected LexicalHandler lexicalHandler = null;

   /**
    * namespace URI for this ContentHandler
    */
   protected static final String namespaceURI =
      "http://www.ananas.org/2001/XM/Walk/Link";

   /**
    * buffer string for link
    */
   protected StringBuffer linkBuffer = null;

   /**
    * stack with prefixes (used for filtering)
    */
   protected Stack prefixStack = new Stack();

   /**
    * prefixes in within the current element
    */
   protected Map currentPrefixes = new HashMap();

   /**
    * Creates a LinkFilter
    * @param messenger Messenger to report errors to
    * @param supervisor selects the right mover for the job
    */
   public LinkFilter(Messenger messenger,MoversSupervisor supervisor)
   {
      this.messenger = messenger;
      this.supervisor = supervisor;
   }

   /**
    * Sets the current directory as well as how far it is from the
    * root (0 = root, 1 = one level below the root, 2 = two levels
    * below the root, etc.)<br />
    * The level cannot be deduced immediately from the directory 'cos
    * we still need to know what the root is
    * Incidentally, the ability to change this must be carefully
    * weighted when running in a multi-thead environment.
    * @param directory
    * @param depth depth from the root
    */
   public void setDirectory(File directory,int depth)
   {
      this.directory = directory;
      StringBuffer prefix = new StringBuffer();
      for(int i = 0;i < depth;i++)
         prefix.append("../");
      rootPrefix = prefix.toString();
   }

   /**
    * Fixes a link.
    * @param link link to fix
    * @return fixed link
    */
   public String fix(String link)
      throws XMException
   {
      // I choose '!' 'cos it's neutral against XSLT (where $
      // indicates variables) and URL (which make frequent use
      // of other good choices such as %, ? and &
      // admitidely, '!' is not a prohibited character for URLs
      // but I don't expect it's used often to start a path...
      // beside one could always escape it as %21
      if(link.startsWith("!"))
         link = rootPrefix + link.substring(1);
      int pos = link.indexOf('#');
      if(pos == 0)
         // the link consists only of a ref, returns immediately
         return link;
      String ref = null;
      if(pos != -1)
      {
         ref = link.substring(pos);
         link = link.substring(0,pos);
      }
      File linkFile = null;
      linkFile = new File(directory,link);
      if(linkFile.exists() && linkFile.isFile())
      {
         String outFile = supervisor.getMover(linkFile).getTargetName(linkFile);
         pos = link.lastIndexOf('/');
         if(pos != -1)
            link = link.substring(0,pos + 1) + outFile;
         else
            link = outFile;
      }
      else if(!(linkFile.isDirectory() || link.startsWith("http:") ||
                link.startsWith("ftp:") || link.startsWith("mailto:") ||
                link.startsWith("news:") || link.startsWith("https:")))
         messenger.warning(new XMException(Resources.getString("suspiciouslink"),new Object[] { link }));
      if(ref != null)
         link += ref;
      return link;
   }

   /**
    * intercept the HTML elements we want to fix
    */
   public void startElement(String namespaceURI,
                            String lName,
                            String qName,
                            Attributes attributes)
      throws SAXException
   {
      prefixStack.push(currentPrefixes);
      currentPrefixes = new HashMap();
      if(namespaceURI != null &&
         namespaceURI.equals(this.namespaceURI) &&
         lName.equals("Fix"))
         linkBuffer = new StringBuffer();
      else
      {
         try
         {
            int index = -1;
            if(qName.equals("a") || qName.equals("link"))
               index = attributes.getIndex("href");
            else if(qName.equals("img") || qName.equals("input"))
               index = attributes.getIndex("src");
            else if(qName.equals("td"))
               index = attributes.getIndex("background");
            if(index != -1)   // note that <a name="xxx"> has no href
            {
               String link = attributes.getValue(index);
               link = fix(link);
               if(attributes instanceof AttributesImpl)
                  ((AttributesImpl)attributes).setValue(index,link);
               else
                  throw new NotImplementedException("only works for AttributeListImpl so far");
            }
            for(int i = attributes.getLength() - 1;i >= 0;i--)
               if(attributes.getQName(i).startsWith("xmlns") &&
                  attributes.getValue(i).equals(this.namespaceURI))
                  if(attributes instanceof AttributesImpl)
                     ((AttributesImpl)attributes).removeAttribute(i);
                  else
                     throw new NotImplementedException("only works for AttributeListImpl so far");
         }
         catch(XMException e)
         {
            throw new SAXException(e);
         }
         super.startElement(namespaceURI,lName,qName,attributes);
      }
   }

   /**
    * Intercept links in element contents
    * @param ch the characters from the XML document
    * @param start the start position in the array
    * @param length the number of characters to read from the array
    */
   public void characters(char[] ch,int start,int length)
      throws SAXException
   {
      if(linkBuffer != null)
         linkBuffer.append(ch,start,length);
      else
         super.characters(ch,start,length);
   }

   /**
    * intercept the HTML elements we want to fix
    */
   public void endElement(String namespaceURI,
                          String lName,
                          String qName)
      throws SAXException
   {
      currentPrefixes = (Map)prefixStack.pop();
      if(namespaceURI != null &&
         namespaceURI.equals(this.namespaceURI) &&
         lName.equals("Fix"))
      {
         try
         {
            String link = linkBuffer.toString().trim();
            link = fix(link);
            super.characters(link.toCharArray(),0,link.length());
            linkBuffer = null;
         }
         catch(XMException e)
         {
            throw new SAXException(e);
         }
      }
      else
         super.endElement(namespaceURI,lName,qName);
   }

   /**
    * Begin the scope of a prefix-URI Namespace mapping.
    * @param prefix The Namespace prefix being declared.
    * @param uri The Namespace URI the prefix is mapped to.
    */
   public void startPrefixMapping(String prefix,String uri)
      throws SAXException
   {
      if(uri.equals(namespaceURI))
         currentPrefixes.put(prefix,Boolean.TRUE);
      else
      {
         super.startPrefixMapping(prefix,uri);
         currentPrefixes.put(prefix,Boolean.FALSE);
      }
   }

   /**
    * End the scope of a prefix-URI mapping.
    * @param prefix The prefix that was being mapping.
    */
   public void endPrefixMapping(String prefix)
      throws SAXException
   {
      Boolean toIgnore = (Boolean)currentPrefixes.get(prefix);
      if(toIgnore != null)
      {
         if(toIgnore.booleanValue())
            super.endPrefixMapping(prefix);
      }
      else
         throw new NotImplementedException("uri should never be null");
   }

   /**
    * Set the value of a property.
    * @param name property name
    * @param value property value
    */
   public void setProperty(String name,Object value)
      throws SAXNotRecognizedException, SAXNotSupportedException
   {
      if(name.equals(JAXPHelper.LEXICAL_HANDLER) &&
         value instanceof LexicalHandler)
         this.lexicalHandler = (LexicalHandler)lexicalHandler;
      else
         super.setProperty(name,value);
   }

   /**
    * Get the value of a property.
    * @param name property name
    * @return property value
    */
   public Object getProperty(String name)
      throws SAXNotRecognizedException, SAXNotSupportedException
   {
      if(name.equals(JAXPHelper.LEXICAL_HANDLER))
         return lexicalHandler;
      else
         return super.getProperty(name);
   }

   /**
    * Comment found.
    */
   public void comment(char[] ch,int start,int length)
      throws SAXException
   {
      if(lexicalHandler != null)
         lexicalHandler.comment(ch,start,length);
   }

   /**
    * End of CDATA section.
    */
   public void endCDATA()
      throws SAXException
   {
      if(lexicalHandler != null)
         lexicalHandler.endCDATA();
   }

   /**
    * End of DTD.
    */
   public void endDTD()
      throws SAXException
   {
      if(lexicalHandler != null)
         lexicalHandler.endDTD();
   }

   /**
    * End of entity.
    */
   public void endEntity(String name)
      throws SAXException
   {
      if(lexicalHandler != null)
         lexicalHandler.endEntity(name);
   }

   /**
    * Beginning of CDATA section.
    */
   public void startCDATA()
      throws SAXException
   {
      if(lexicalHandler != null)
         lexicalHandler.startCDATA();
   }

   /**
    * Beginning of DTD.
    */
   public void startDTD(String name,String publicId,String systemId)
      throws SAXException
   {
      if(lexicalHandler != null)
         lexicalHandler.startDTD(name,publicId,systemId);
   }

   /**
    * Beginning of entity.
    */
   public void startEntity(String name)
      throws SAXException
   {
      if(lexicalHandler != null)
         lexicalHandler.startEntity(name);
   }
}
