/**
 * Style the file it copies using an XSLT style sheet. There's
 * a dependancy on <a href="http://xml.apache.org">Xalan</a>
 * because we use Xalan serializer.
 *
 * @author <a href="http://www.marchal.com">Beno&#238;t Marchal</a>
 */

// Copyright 2001, Benot Marchal.

package org.ananas.xm;

import java.io.*;
import java.util.*;
import org.xml.sax.*;
import javax.xml.transform.*;
import javax.xml.transform.sax.*;
import org.ananas.util.JAXPHelper;
import javax.xml.transform.stream.*;
import org.apache.xalan.serialize.*;
import org.xml.sax.ext.LexicalHandler;
import org.ananas.util.NotImplementedException;
import org.apache.xalan.templates.OutputProperties;

public class StylingMover
   extends DefaultMoverImpl
{
   /**
    * "compiled" style sheet
    */
   protected TemplatesManager templates;

   /**
    * time that the style sheet was last modified
    */
   protected long lastModified;

   /**
    * the link filter we use to process links
    */
   protected LinkFilter linkFilter;

   /**
    * XMLFilters to use while styling these documents
    */
   protected XMLFilter xmlFilterWithDTD,
                       xmlFilterWithoutDTD;

   /**
    * XMLReader used to extract the processing instructions
    */
   protected XMLReader xmlReader;

   /**
    * processing instruction handler to extract the processing instructions
    */
   protected ProcessingInstructionHandler processingInstructionHandler;

   /**
    * Creates a StylingMover from an XSLT stylesheet file.
    * @param messenger Messenger to report errors to
    * @param supervisor the MoversSupervisor used in this application
    * @param templates stylesheet files
    * @param xmlFilterWithDTD pre-process the documents through this filter
    *    this one processes the external DTD if present
    * @param xmlFilterWithoutDTD pre-process the documents through this filter
    *    this one ignores the external DTD if present
    * @param build perform a build, i.e. process all files
    */
   public StylingMover(Messenger messenger,
                       MoversSupervisor supervisor,
                       TemplatesManager templates,
                       XMLFilter xmlFilterWithDTD,
                       XMLFilter xmlFilterWithoutDTD,
                       boolean build)
      throws XMException
   {
      super(messenger,build);
      try
      {
         linkFilter = new LinkFilter(messenger,supervisor);
         this.templates = templates;
         this.xmlFilterWithDTD = xmlFilterWithDTD;
         this.xmlFilterWithoutDTD = xmlFilterWithoutDTD;
         xmlReader = JAXPHelper.createXMLReader(false,false);
         processingInstructionHandler = new ProcessingInstructionHandler();
         xmlReader.setContentHandler(processingInstructionHandler);
      }
      catch(SAXException e)
      {
         throw new XMException(e);
      }
   }

   /**
    * Creates a StylingMover from an XSLT stylesheet file.
    * No pre-processing through a filter.
    * @param messenger Messenger to report errors to
    * @param supervisor the MoversSupervisor used in this application
    * @param templates stylesheet files
    * @param build perform a build, i.e. process all files
    */
   public StylingMover(Messenger messenger,
                       MoversSupervisor supervisor,
                       TemplatesManager templates,
                       boolean build)
      throws XMException
   {
      this(messenger,supervisor,templates,null,null,build);
   }

  /**
    * Return the name the file would have after moving. In this case,
    * it just changes the extension.
    * @param file file the mover would have to move
    * @return the extension it would give the file in the target directory
    */
   public String getTargetName(File file)
      throws XMException
   {
      try
      {
         xmlReader.parse(JAXPHelper.toInputSource(file));
      }
      catch(SAXException e)
      {
         if(!processingInstructionHandler.isStopException(e))
            throw new XMException(e);
      }
      catch(IOException e)
      {
         throw new XMException(e);
      }

      String href = processingInstructionHandler.getHRef();
      // set the second parameter to null or it might change the
      // source document object (which we do not want to)
      // it's another testimonial to the fact that I need to
      // refactor how the whole app works
      Transformer transformer = templates.getTransformer(href,null);
      String extension = transformer.getOutputProperty("{http://ananas.org/2001/XM/XSLT/Rules}extension");
      if(extension == null)
         extension = transformer.getOutputProperty("method");
      return getDisplayName(file) + "." + extension;
   }

   /**
    * Style one file.
    * @param sourceFile the file to style
    * @param targetDir the directory to copy the result to
    * @param depth how far from the root (0 = root)
    * @return styled file or null if the file was ignored
    */
   public synchronized File move(File sourceFile,File targetDir,int depth)
      throws IOException, XMException
   {
      try
      {
         File targetFile = new File(targetDir,getTargetName(sourceFile));
         if(!build && xmlFilterWithDTD == null &&
            xmlFilterWithoutDTD == null && targetFile.exists())
            if(targetFile.lastModified() >= sourceFile.lastModified())
               return null;
         linkFilter.setDirectory(sourceFile.getParentFile(),depth);
         SAXResult result = new SAXResult(linkFilter);
         // sets a base output which Xalan redirect extension uses...
         // (it places new files in the same directory as the current one)
         result.setSystemId(targetFile.toURL().toExternalForm());

         try
         {
            xmlReader.parse(JAXPHelper.toInputSource(sourceFile));
         }
         catch(SAXException e)
         {
            if(!processingInstructionHandler.isStopException(e))
               throw e;
         }

         String href = processingInstructionHandler.getHRef();
         Transformer transformer = templates.getTransformer(href,sourceFile);
         Iterator iterator = processingInstructionHandler.getParamsIterator();
         while(iterator.hasNext())
         {
            Map.Entry entry = (Map.Entry)iterator.next();
            transformer.setParameter((String)entry.getKey(),(String)entry.getValue());
         }
         Source source = null;
         if(xmlFilterWithDTD != null && xmlFilterWithoutDTD != null)
         {
            XMLFilter xmlFilter = processingInstructionHandler.isLoadExternalDTD() ?
               xmlFilterWithDTD : xmlFilterWithoutDTD;
            try
            {
               xmlFilter.setProperty(SOURCE_FILE,sourceFile);
            }
            // it's OKay if it does not recognize the property, it
            // just means it's not one of our filter
            catch(SAXNotRecognizedException e)
               { }
            source = new SAXSource(xmlFilter,JAXPHelper.toInputSource(sourceFile));
         }
         else
            source = new SAXSource(JAXPHelper.createXMLReader(true,processingInstructionHandler.isLoadExternalDTD()),
                                   JAXPHelper.toInputSource(sourceFile));

         // the lexical handler is required for CDATA sections
         Serializer serializer = SerializerFactory.getSerializer(transformer.getOutputProperties());
         serializer.setOutputStream(new FileOutputStream(targetFile));
         linkFilter.setContentHandler(serializer.asContentHandler());
         if(serializer instanceof LexicalHandler)
            linkFilter.setProperty(JAXPHelper.LEXICAL_HANDLER,serializer);
         else
            throw new NotImplementedException("The serializer has to implement the LexicalHandler interface");

/*         // new -- to test the redirect
         transformer.setErrorListener(new ErrorListener()
         {
            public void error(TransformerException x)
               throws TransformerException
            {
               try
               {
                  PrintWriter fw = new PrintWriter(new FileWriter("debug.txt",true));
                  fw.println();
                  x.printStackTrace(fw);
                  fw.println();
                  fw.close();
               }
               catch(IOException e)
               {
                  throw new TransformerException(e.getMessage());
               }
            }

            public void fatalError(TransformerException x)
               throws TransformerException
            {
               try
               {
                  PrintWriter fw = new PrintWriter(new FileWriter("debug.txt",true));
                  fw.println();
                  x.printStackTrace(fw);
                  fw.println();
                  fw.close();
               }
               catch(IOException e)
               {
                  throw new TransformerException(e.getMessage());
               }
               throw x;
            }

            public void warning(TransformerException x)
               throws TransformerException
            {
               try
               {
                  PrintWriter fw = new PrintWriter(new FileWriter("debug.txt",true));
                  fw.println();
                  x.printStackTrace(fw);
                  fw.println();
                  fw.close();
               }
               catch(IOException e)
               {
                  throw new TransformerException(e.getMessage());
               }
            }
         });
         // end of new*/


         transformer.transform(source,result);
         return targetFile;
      }
      catch(SAXException e)
      {
         messenger.error(new XMException(e));
      }
      catch(TransformerException e)
      {
         messenger.error(new XMException(e));
      }
    
      return null;
   }
}
