package org.ananas.xm.eclipse;
 
import java.io.*;
import org.xml.sax.*; 
import org.ananas.xm.*;
import org.eclipse.ui.*;
import org.eclipse.swt.SWT;
import javax.xml.transform.*;
import org.eclipse.ui.part.*;
import java.text.MessageFormat;
import org.eclipse.swt.events.*;
import org.eclipse.swt.widgets.*;

// TODO : future additions: actions to clear the log, etc.
// menus and buton icons on the bar
// XM own editor

/**
 * Implements an XM messenger as an Eclipse part. In other words,
 * shows XM messages in the Eclipse workspace.
 *
 * @author bmarchal
 */
public class MessengerView
   extends ViewPart 
   implements Messenger
{
   /**
    * On-screen control.
    */
   private Table table;
	
   /**
    * Message to create a location...
    */
   private static MessageFormat locationFormat;
	
   /**
    * This class updates the table (add an item) in a separate thread
    * (typically used to synchronized with the UI thread).
    * It creates a new instance of this class for every update: not the most
    * efficient but it does guarantee concurrent execution.
    */
   private class AddItem
      implements Runnable
   {
      private String[] text;
      public AddItem(String type,String message,String location)
      {
         text = new String[] { type, message, location };
      }
      public void run()
      {
         TableItem item = new TableItem(table,SWT.NONE);
         item.setText(text);
      }
   }

   /**
    * This class adds the console (an instance of MessengerView)
    * to the workbench in a separate thread. Typically used to synchronize
    * with the UI thread.
    */
   private static class ShowConsole
      implements Runnable
   {
      private IWorkbench workbench;
      private MessengerView console;
      public ShowConsole(IWorkbench workbench)
      {
         this.workbench = workbench;
      }
      public MessengerView getConsole()
      {
         return console;
      }
      public void run()
      {
         try
         {
            IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
            if(window == null)
            {
               IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
               if(windows != null && windows.length > 0)
                  window = windows[0];
               else
                  return;
            }
            IWorkbenchPage page = window.getActivePage();
            if(page != null)
               console = (MessengerView)page.showView(PluginConstants.CONSOLE_ID);
         }
         catch(PartInitException x)
            { }
      }
   }
	
   /**
    * Creates a new MessengerView
    */
   public MessengerView()
   {
      super();
      locationFormat = new MessageFormat(Resources.getString("eclipse.location"));
   }

   /**
    * Eclipse API: creates the control for this part.
    */
   public void createPartControl(Composite parent)
   {
      table = new Table(parent,SWT.SINGLE);
      TableColumn column = new TableColumn(table,SWT.NONE);
      column.setText(Resources.getString("eclipse.status"));
      column.setWidth(50);
      column = new TableColumn(table,SWT.NONE);
      column.setText(Resources.getString("eclipse.message"));
      column.setWidth(500);
      column = new TableColumn(table,SWT.NONE);
      column.setText(Resources.getString("eclipse.location"));
      column.setWidth(75);
      table.setHeaderVisible(true);
      table.addSelectionListener(new SelectionListener()
      {
         public void widgetDefaultSelected(SelectionEvent e)
            { updateTooltip(); }
         
         public void widgetSelected(SelectionEvent e)
            { updateTooltip(); }
      });
   }

   /**
    * Changes the table tooltip to the current message (the
    * content of the first column). It's useful to read long messages.
    */
   protected void updateTooltip()
   {
      TableItem[] items = table.getSelection();
      if(items != null)
         table.setToolTipText(items[0].getText(1));
      else
         table.setToolTipText(null);
   }

   /**
    * Eclipse API: sets the focus on this part.
    */
   public void setFocus()
   {
      if(table != null)
         table.setFocus();
   }

   /**
    * Adds a new item (a line) to the message table. It takes care to synchronize
    * with the UI thread.
    * @param type first column (message type)
    * @param message second column (message itself)
    * @param location last column (message location)
    * @throws XMException
    */
   private void addItem(String type,String message,String location)
   {
      if(table != null)
      {
         Display display = Display.getCurrent();
         if(display == null)
            display = Display.getDefault();
         display.asyncExec(new AddItem(type,message,location));
      }
      else
         throw new NullPointerException("Table control has not been initialized in MessengerView");
   }

   /**
    * Utility method: adds an exception to the window.
    * @param type describes the type of error (fatal, error)
    * @param x describes the problem
    */
   private void addItem(String type,XMException x)
      throws XMException
   {
      if(table != null)
      {
         Throwable t = PluginTools.popThrowables(x);
         String location = locationAsString(t);
         addItem(type,t.getLocalizedMessage(),location);
      }
      else
         throw x;
   }

   /**
    * @see org.ananas.xm.Messenger#error(XMException)
    */
   public void error(XMException x)
      throws XMException
   {
      addItem(Resources.getString("error"),x);
   }

   /**
    * @see org.ananas.xm.Messenger#fatal(XMException)
    */
   public void fatal(XMException x)
      throws XMException
   {
      addItem(Resources.getString("fatal"),x);
   }

   /**
    * @see org.ananas.xm.Messenger#warning(XMException)
    */
   public void warning(XMException x)
      throws XMException
   {
      addItem(Resources.getString("warning"),x);
   }

   /**
    * @see org.ananas.xm.Messenger#progress(File, File)
    */
   public boolean progress(File sourceFile,File resultFile)
      throws XMException
   {
      if(resultFile != null)
         info(Resources.getString("progressdone"),
              new Object[] { sourceFile.getPath(), resultFile.getPath() });
      else
         info(Resources.getString("progressignored"),
              new Object[] { sourceFile.getPath() });
      return true;
   }

   /**
    * @see org.ananas.xm.Messenger#info(String)
    */
   public void info(String msg)
      throws XMException
   {
      if(table != null)
         addItem(Resources.getString("info"),msg,null);
   }

   /**
    * @see org.ananas.xm.Messenger#info(String, Object[])
    */
   public void info(String pattern, Object[] arguments)
      throws XMException
   {
      info(MessageFormat.format(pattern,arguments));
   }

   /**
    * Clears all messages displayed so far.
    */
   public void clean()
   {
      if(table != null)
      {
         Display display = Display.getCurrent();
         if(display == null)
            display = Display.getDefault();
         display.syncExec(new Runnable()
         {
            public void run()
            {
               table.removeAll();
            }
         });
      }
   }

   /**
    * Helper method: it attempts to extract location information from
    * a Throwable (e.g. typecast to SAXParseException and similar)
    * @param t exception to process
    * @return location as string if available, null otherwise
    */
   private static String locationAsString(Throwable t)
   {
      if(t instanceof SAXParseException)
      {
         SAXParseException x = (SAXParseException)t;
         return locationFormat.format(new Object[]
         {
         	extractFilename(x.getSystemId()),
            new Integer(x.getColumnNumber()),
            new Integer(x.getLineNumber()),
         });
      }
      else if(t instanceof TransformerException)
      {
         TransformerException x = (TransformerException)t;
         SourceLocator locator = x.getLocator();
         return locationFormat.format(new Object[]
         {
         	extractFilename(locator.getSystemId()),
            new Integer(locator.getColumnNumber()),
            new Integer(locator.getLineNumber()),
         });
      }
      else
         return null;
   }

   /**
    * Helper method: extracts the filename from the system id.
    * @param systemID system id to extract the filename from
    * @return filename, if possible
    */
   private static String extractFilename(String systemID)
   {
   	  char[] separators = new char[]
   	  {
   	     File.separatorChar, '/', '\\',
   	  };
   	  for(int i = 0;i < separators.length;i++)
   	  {
   	     int pos = systemID.lastIndexOf(separators[i]);
   	     if(pos != -1)
   	        return systemID.substring(pos + 1);
   	  }
   	  return systemID == null ? "?" : systemID;
   }

   /**
    * Returns the XM Console, i.e. the shared instance of MessengerView that
    * is currently on screen. If none is on-screen, it brings one up.
    * This method synchronizes with the UI thread.
    * @param workbench the workbench in use
    * @return shared XM console
    */
   public static MessengerView showConsole(IWorkbench workbench)
   {
      Display display = Display.getCurrent();
      if(display == null)
         display = Display.getDefault();
      ShowConsole showConsole = new ShowConsole(workbench);
      display.syncExec(showConsole);
      return showConsole.getConsole();
   }
}