package org.ananas.xm.eclipse;

import java.io.*;
import org.eclipse.ui.*;
import org.ananas.xm.Resources;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.dialogs.*;
import org.eclipse.core.resources.*;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.resource.ImageDescriptor;
import java.lang.reflect.InvocationTargetException;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;
 
/**
 * Creates a new XM project in the Eclipse workbench.
 * Populates the project with default files.
 * 
 * @author bmarchal
 */
public class NewXMProjectWizard
   extends Wizard
   implements INewWizard
{
   /**
    * The main page on the wizard: collects the project name and location.
    */
   private WizardNewProjectCreationPage namePage;

   /**
    * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
    */
   public void init(IWorkbench workbench,IStructuredSelection selection)
   {
      setNeedsProgressMonitor(true);
   }

   /**
    * @see org.eclipse.jface.wizard.IWizard#addPages()
    */
   public void addPages()
   {
   	try
   	{
      super.addPages();
      namePage = new WizardNewProjectCreationPage("NewXMProjectWizard");
      namePage.setTitle(Resources.getString("eclipse.newprojectname"));
      namePage.setDescription(Resources.getString("eclipse.newprojectdescription"));
      namePage.setImageDescriptor(ImageDescriptor.createFromFile(getClass(),
                                  "/org/ananas/xm/eclipse/resources/newproject58.gif"));
      addPage(namePage);
   	}
   	catch(Exception x)
   	{
   		reportError(x);
   	}
   }

   /**
    * User has clicked "Finish", we create the project.
    * In practice, it calls the createProject() method in the appropriate thread.
    * @see #createProject(IProjectMonitor)
    * @see org.eclipse.jface.wizard.IWizard#performFinish()
    */
   public boolean performFinish()
   {
      try
      {
         getContainer().run(false,true,new WorkspaceModifyOperation()
         {
            protected void execute(IProgressMonitor monitor)
            {
               createProject(monitor != null ? monitor : new NullProgressMonitor());
            }
         });
      }
      catch(InvocationTargetException x)
      {
         reportError(x);
         return false;
      }
      catch(InterruptedException x)
      {
         reportError(x);
         return false;
      }
      return true; 
   }

   /**
    * This is the actual implementation for project creation.
    * @param monitor reports progress on this object
    */
   protected void createProject(IProgressMonitor monitor)
   {
      monitor.beginTask(Resources.getString("eclipse.creatingproject"),50);
      try
      {
         IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
         monitor.subTask(Resources.getString("eclipse.creatingdirectories"));
         IProject project = root.getProject(namePage.getProjectName());
         IProjectDescription description = ResourcesPlugin.getWorkspace().newProjectDescription(project.getName());
         if(!Platform.getLocation().equals(namePage.getLocationPath()))
            description.setLocation(namePage.getLocationPath());
         description.setNatureIds(new String[] { PluginConstants.NATURE_ID });
         ICommand command = description.newCommand();
         command.setBuilderName(PluginConstants.BUILDER_ID);
         description.setBuildSpec(new ICommand[] { command });
         project.create(description,monitor);
         monitor.worked(10);
         project.open(monitor);
         project.setPersistentProperty(PluginConstants.SOURCE_PROPERTY_NAME,"src");
         project.setPersistentProperty(PluginConstants.RULES_PROPERTY_NAME,"rules");
         project.setPersistentProperty(PluginConstants.PUBLISH_PROPERTY_NAME,"publish");
         project.setPersistentProperty(PluginConstants.BUILD_PROPERTY_NAME,"false");
         monitor.worked(10);
         IPath projectPath = project.getFullPath(),
               srcPath = projectPath.append("src"),
               rulesPath = projectPath.append("rules"),
               publishPath = projectPath.append("publish");
         IFolder srcFolder = root.getFolder(srcPath),
                 rulesFolder = root.getFolder(rulesPath),
                 publishFolder = root.getFolder(publishPath);
         createFolderHelper(srcFolder,monitor);
         createFolderHelper(rulesFolder,monitor);
         createFolderHelper(publishFolder,monitor);
         monitor.worked(10);
         monitor.subTask(Resources.getString("eclipse.creatingfiles"));
         IPath indexPath = srcPath.append("index.xml"),
               defaultPath = rulesPath.append("default.xsl");
         IFile indexFile = root.getFile(indexPath),
               defaultFile = root.getFile(defaultPath);
         Class clasz = getClass();
         InputStream indexIS = clasz.getResourceAsStream("/org/ananas/xm/eclipse/resources/index.xml"),
                     defaultIS = clasz.getResourceAsStream("/org/ananas/xm/eclipse/resources/default.xsl");
         indexFile.create(indexIS,false,new SubProgressMonitor(monitor,10));
         defaultFile.create(defaultIS,false,new SubProgressMonitor(monitor,10));
      }
      catch(CoreException x)
      {
         reportError(x);
      }
      finally
      {
         monitor.done();
      }
   }

   /**
    * Displays an error that occured during the project creation.
    * @param x details on the error
    */
   private void reportError(Exception x)
   {
      ErrorDialog.openError(getShell(),
                            Resources.getString("eclipse.dialogtitle"),
                            Resources.getString("eclipse.projecterror"),
                            PluginTools.makeStatus(x));
   }

   /**
    * Helper method: it recursively creates a folder path.
    * @param folder
    * @param monitor
    * @throws CoreException
    * @see java.io.File#mkdirs()
    */
   private void createFolderHelper(IFolder folder,IProgressMonitor monitor)
      throws CoreException
   {
      if(!folder.exists())
      {
         IContainer parent = folder.getParent();
         if(parent instanceof IFolder
            && (!((IFolder)parent).exists()))
            createFolderHelper((IFolder)parent,monitor);
         folder.create(false,true,monitor);
      }
   }
}