package org.ananas.xm.eclipse;

import java.io.*;
import java.util.Map;
import org.ananas.xm.*;
import org.eclipse.ui.*;
import org.eclipse.core.runtime.*;
import org.eclipse.core.resources.*;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.widgets.Display;

/**
 * Implements the builder interface for XM.
 * This is required so Eclipse can build the project from the "Project" menu.
 *
 * @author bmarchal
 */
public class XMBuilder
   extends IncrementalProjectBuilder
{
   /**
    * Retains a reference to the workbench that we can use throughout the build.
    */
   private IWorkbench workbench;
   
   /**
    * Creates a new XMBuilder.
    */
   public XMBuilder()
   {
      workbench = PlatformUI.getWorkbench();
   }
	
   /**
    * Builds the project.
    * @see org.eclipse.core.internal.events.InternalBuilder#build(int, Map, IProgressMonitor)
    */
   protected IProject[] build(int kind,Map args,IProgressMonitor _monitor)
   {
      try
      {
         IProject project = getProject();
         if(project != null && project.isAccessible())
         {
            if(_monitor == null)
               _monitor = new NullProgressMonitor();
            saveDirtyEditors();
            MessengerProxy proxy = new MessengerProxy();
            MessengerView console = MessengerView.showConsole(workbench);
            if(console != null)
            {
               console.clean();
               proxy.addMessenger(console);
            }
            MessengerMonitor monitor = new MessengerMonitor(_monitor,"Publish XML documents");
            proxy.addMessenger(monitor);
            runXM(proxy);
            project.refreshLocal(25,_monitor);
            monitor.done();
         }
      }
      catch(Exception x)
      {
         ErrorDialog.openError(workbench.getActiveWorkbenchWindow().getShell(),
                               Resources.getString("eclipse.dialogtitle"),
                               Resources.getString("eclipse.builderror"),
                               PluginTools.makeStatus(x));
      }
      return new IProject[0]; 
   }

   /**
    * Runs XM. Simply launches the XM product to publish the site.
    * Note that XM does not rely on Eclipse to decide which file has
    * been modified, etc.
    * @param messenger messenger to report errors on
    * @throws IOException I/O error, e.g. accessing files
    * @throws XMException XM reports an error
    */
   private void runXM(Messenger messenger)
      throws CoreException, IOException, XMException
   {
      IProject project = getProject();
      String rulesPath = project.getPersistentProperty(PluginConstants.RULES_PROPERTY_NAME),
             sourcePath = project.getPersistentProperty(PluginConstants.SOURCE_PROPERTY_NAME),
             publishPath = project.getPersistentProperty(PluginConstants.PUBLISH_PROPERTY_NAME);
      if(rulesPath == null)
         rulesPath = "rules";
      if(sourcePath == null)
         sourcePath = "src"; 
      if(publishPath == null)
         publishPath = "publish";
      rulesPath = project.getFolder(rulesPath).getLocation().toOSString();
      sourcePath = project.getFolder(sourcePath).getLocation().toOSString();
      publishPath = project.getFolder(publishPath).getLocation().toOSString();
      DirectoryWalker walker = new DirectoryWalker(messenger,rulesPath,false);
      walker.walk(sourcePath,publishPath);
   }

   /**
    * Saves dirty editors (i.e. those editors that were modified but
    * not saved) in the workbench. It takes care to run the saving process
    * in the UI thread.
    */
   private void saveDirtyEditors()
   {
      Display display = Display.getCurrent();
      if(display == null)
         display = Display.getDefault();
      display.syncExec(new Runnable()
      {
         public void run()
         {
            IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
            for(int i = 0;i < windows.length;i++)
            {
               IWorkbenchPage[] pages = windows[i].getPages();
               for(int j = 0;j < pages.length;j++)
                   pages[j].saveAllEditors(false);
            }
         }
      });
   }
}